# -*- coding: cp949 -*-
#
# Copyright (C) 2011 by Woosuk Suh
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import wx
import os
import sys
import thread
import urllib

from twisted.internet import defer
from twisted.internet import reactor

from searchengine import SearchEngine
from grabdownloader import GrabDownloader


WINDOW_TITLE = "Gelbooru Grabber"


class GrabberFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, wx.ID_ANY, WINDOW_TITLE, 
                style=wx.DEFAULT_FRAME_STYLE & ~(wx.MAXIMIZE_BOX))
        self.panel = wx.Panel(self, wx.ID_ANY)
        searchLabel = wx.StaticText(self.panel, wx.ID_ANY, "Type tags just like you do in Gelbooru.")
        self.searchText = wx.TextCtrl(self.panel, wx.ID_ANY, "", size=(400, -1))
        self.downloadButton = wx.Button(self.panel, wx.ID_ANY, "Download")
        self.exitButton = wx.Button(self.panel, wx.ID_ANY, "Exit")
        self.Bind(wx.EVT_BUTTON, self.onDownload, self.downloadButton)
        self.Bind(wx.EVT_BUTTON, self.onExit, self.exitButton)

        statusLabel = wx.StaticText(self.panel, wx.ID_ANY, "Status displayed below:")
        errorLabel = wx.StaticText(self.panel, wx.ID_ANY, "Errors displayed below:")
        self.statusText = wx.TextCtrl(self.panel, wx.ID_ANY,
                size=(-1, 100),
                style=wx.TE_MULTILINE|wx.TE_READONLY)
        self.errorText = wx.TextCtrl(self.panel, wx.ID_ANY,
                size=(-1, 100),
                style=wx.TE_MULTILINE|wx.TE_READONLY)

        grandSizer = wx.BoxSizer(wx.VERTICAL)
        searchSizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        statusSizer = wx.BoxSizer(wx.VERTICAL)
        
        searchSizer.Add(self.searchText, 0, wx.ALL, 5)

        buttonSizer.Add(self.downloadButton, 0, wx.ALL, 5)
        buttonSizer.Add(self.exitButton, 0, wx.ALL, 5)

        statusSizer.Add(statusLabel, 0, wx.LEFT)
        statusSizer.Add(self.statusText, 0, wx.CENTER|wx.EXPAND)
        statusSizer.Add(errorLabel, 0, wx.LEFT)
        statusSizer.Add(self.errorText, 0, wx.CENTER|wx.EXPAND)

        grandSizer.Add(searchLabel, 0, wx.CENTER)
        grandSizer.Add(searchSizer, 0, wx.CENTER)
        grandSizer.Add(buttonSizer, 0, wx.CENTER)
        grandSizer.Add(statusSizer, 0, wx.CENTER|wx.EXPAND)
        self.panel.SetSizer(grandSizer)
        grandSizer.Fit(self)

        self.prepareCores()

    def prepareCores(self):
        self.gd = GrabDownloader(ui=self)

    def onDownload(self, evt):
        value = self.searchText.GetValue().strip()
        value = urllib.quote_plus(value)
        if not value:
            self.updateError("Please input the tag value!")
            return
        self.downloadButton.Enable(False)
        self.searchText.Enable(False)
        self.updateStatus("Begin searching %s... This may take up some time." % value)
        self.gd.update_tags(value)

        thread.start_new_thread(self.gd.start_download, tuple())

    def onExit(self, evt):
        self.Destroy()

    def updateStatus(self, text):
        self.statusText.WriteText(text)
        self.statusText.WriteText("\n")

    def updateError(self, text):
        self.errorText.WriteText(text)
        self.errorText.WriteText("\n")


def start_reactor(**kwargs):
    sys.exit(reactor.run(**kwargs))


if __name__ == "__main__":
    app = wx.PySimpleApp()
    frame = GrabberFrame()
    frame.Show()
    thread.start_new_thread(start_reactor, tuple(), {"installSignalHandlers": 0})
    app.MainLoop()
